

plot_xy_hist <- function(veclist, x, y, nbars=5, force=0, xscale=1, yscale=1,
                  xlab="", ylab="", title="", col="black") {

  myhist <- function(vec, nbars, force=0) {

    ret <- hist(vec, breaks=nbars, plot=FALSE)
    if (!force) return(ret)
    if (length(ret$intensities) == nbars) return(ret)
    qq  <- quantile(vec, probs=seq(0, 1, 1/nbars))
    ret <- hist(vec, breaks=qq, plot=FALSE) 

    ret
  }

  n <- length(veclist)
  if (length(x) != n) stop("ERROR: Lengths of veclist and x must match")
  if (length(y) != n) stop("ERROR: Lengths of veclist and y must match")
  if (!all(is.finite(x))) stop("ERROR: x must only contain finite values")
  if (!all(is.finite(y))) stop("ERROR: y must only contain finite values")
  if (length(nbars) == 1) nbars <- rep(nbars, n)
  if (length(force) == 1) force <- rep(force, n)
  if (length(col) == 1) col <- rep(col, n)
  if (length(nbars) != n) stop("ERROR: Lengths of x and nbars must match")
  if (length(force) != n) stop("ERROR: Lengths of x and force must match")
  if (length(col) != n) stop("ERROR: Lengths of x and col must match")

  histlist <- list()
  max.intensity <- -1
  for (i in 1:n) {
    ret <- myhist(veclist[[i]], nbars[i], force=force[i])
    max <- max(ret$intensities)
    if (max > max.intensity) max.intensity <- max
    histlist[[i]] <- ret
  }
  #print(max.intensity)

  xlim <- c(min(x, na.rm=TRUE), max(x, na.rm=TRUE))
  ylim <- c(min(y, na.rm=TRUE), max(y, na.rm=TRUE))
  p1   <- sum(xlim)/2
  p2   <- sum(ylim)/2
  xlim[1] <- xlim[1] - xscale
  xlim[2] <- xlim[2] + 2*xscale
  ylim[1] <- ylim[1] - yscale
  ylim[2] <- ylim[2] + 2*yscale

  # Set up the axes, plot nothing
  plot(p1, p2, type="n", xlim=xlim, ylim=ylim, 
       xlab=xlab, ylab=ylab, main=title)
  
  # Plot each histogram at the (x,y) coordinate
  for (i in 1:n) {
    int     <- histlist[[i]]$intensities
    nint    <- length(int)
    h       <- xscale/nint
    x0      <- h*(0:nint)
    x2      <- x[i] + x0
    ybottom <- y[i]
    y2      <- ybottom + (int/max.intensity)*yscale
    # Plot each bar of the histogram
    for (j in 1:nint) {
      xleft  <- x2[j]
      xright <- x2[j+1]
      ytop   <- y2[j]       
      rect(xleft, ybottom, xright, ytop, col=col[i])
    }
  }

  histlist

} 

#set.seed(123)
#nr <- 10000
#nc <- 50
#dlist <- list()
#for (i in 1:nc) dlist[[i]] <- rnorm(ceiling(1000+nr*runif(1)))
#x <- 2*runif(nc) + 10*runif(nc)
#y <- 2*runif(nc) + 5*runif(nc)
#
#ret <- plot_xy_hist(dlist, x, y, nbars=10, xscale=0.25, yscale=0.5,
#              force=0)
#
#f <- "K:/bayesian/results/apr08_2011/plot.jpeg"
#savePlot(file=f, type="jpeg")

