post_badge <- function(geno.mat, data, cc.var, exposure.var, group.var, out.dir, op=NULL) {

  writePlot <- function(prefix) {
    #if (.Platform$OS.type == "windows") {
    #  outfile <- paste(out.dir, op$out.string, prefix, ".jpeg", sep="") 
    #  savePlot(file=outfile, type="jpeg")
    #} else {
    #  outfile <- paste(out.dir, op$out.string, prefix, ".pdf", sep="") 
    #  pdf(file=outfile) 
    #}
    outfile <- paste(out.dir, op$out.string, prefix, ".pdf", sep="") 
    pdf(file=outfile) 
  } 

  op <- default.list(op, c("M1", "M2", "everyN", "out.string", "ncolors"), 
                     list(1, Inf, 1, "", 20))
  M1 <- op$M1
  M2 <- op$M2
  everyN <- op$everyN

  ## get the PC for each subjects
  pca.out <- princomp(x=geno.mat, cor=T)
  pc.mat  <- predict(pca.out)
 
  out.dir    <- checkForSep(out.dir)
  str        <- op$out.string
  alpha.file <- paste(out.dir, str, "out_alpha.txt", sep="")
  beta.file  <- paste(out.dir, str, "out_beta.txt", sep="")
  dev.file   <- paste(out.dir, str, "out_dev_i.txt", sep="")
  z.file     <- paste(out.dir, str, "out_alloc_z.txt", sep="")
  tao.file   <- paste(out.dir, str, "out_tao.txt", sep="")
  psi.file   <- paste(out.dir, str, "out_psi.txt", sep="")  

  # Create a temporary file for the data
  covars <- op[["covars", exact=TRUE]]
  covars <- getAllVars(covars)
  vars   <- c(cc.var, exposure.var, group.var, covars)
  data   <- data[, vars]
  data[, group.var] <- as.numeric(data[, group.var]) - 1
  tempFile <- paste(out.dir, str, "TEMP_DATA.txt", sep="")  
  write.table(data, file=tempFile, sep=" ", row.names=FALSE, col.names=FALSE, quote=FALSE) 

  # Get the number of clusters
  temp <- scan(alpha.file, nlines=1, what="character", sep=" ")
  n.cluster <- length(temp)

  ret.1 <- postProcess(tempFile, z.file, psi.file, alpha.file, beta.file, dev.file, tao.file,
                n.cluster,  M1, M2, NULL, everyN)
  M2 <- ret.1$M2
  dic.value <- 2 * ret.1$mean.dev - ret.1$dev.mean


  ## get the cluster assignment
  sim.mat    <- ret.1$sim.mat
  sim.mat    <- as.matrix(sim.mat)
  dis.mat    <- sim.to.dis(sim.mat) 
  clust.out  <- pam(dis.mat, n.cluster)
  assign.vec <- clust.out$clustering

  ck.id      <- c(1:length(assign.vec))
  grp.list   <- list()
  for (i in 1:n.cluster) grp.list[[i]] <- ck.id[assign.vec==i]

  group.id    <- data[, group.var] + 1
  subj.assign <- rep(-9, dim(data)[1])
  for (i in 1:n.cluster) subj.assign[is.element(group.id, grp.list[[i]])] <- i



  ## plot the cluster assignment at the top 2 pc space
  writePlot("plots")
  plot(pc.mat[,1], pc.mat[,2], xlab="PC.1", ylab="PC.2")
  cls <- rainbow(n.cluster)
  for (i in 1:n.cluster) {
    points(pc.mat[subj.assign==i, c(1,2)], col=cls[i])
  }
  #dev.off()
  
  ## Get the posteior median for the coefficient alpha and beta for each subject
  aa <- table(group.id)
  my.id.vec <- as.numeric(names(aa))
  geno.id   <- sort(x=as.numeric(aa), decreasing=T, index.return=T)$ix[1]
  ret.1     <- process.coeff.id(tempFile, z.file, alpha.file, beta.file, M1, M2,
                    everyN, assign.vec, geno.id)
  file.remove(tempFile)

  exp.alpha.med <- ret.1$alpha.median
  exp.beta.med  <- ret.1$beta.median

  ## rescale exp.alpha into OR

  a.min <- min(exp.alpha.med, na.rm=TRUE)
  alpha.med.odds <- exp.alpha.med/a.min
  beta.med.odds  <- exp.beta.med

  ## Since all subjects from the same genotype group have the same estimate, we just need
  ## to pick one from each genotype group.
  genogrp.list <- list()
  n.grp        <- length(assign.vec)
  n.subj       <- length(subj.assign)
  ck.id        <- c(1:n.subj)
  my.center.id <- rep(0, n.subj)
  for (i in 1: n.grp) {
    genogrp.list[[i]] <- ck.id[group.id==i]
    my.center.id[genogrp.list[[i]][1]] <- 1
  }

  my.mat   <- pc.mat[my.center.id >0, ]
  my.theta <- mean(dist(my.mat))
  hc       <- heat.colors(op$ncolors)
  hc       <- hc[length(hc):1]

  nc <- min(ncol(pc.mat), 5)

  ## the smooth plot for alpha
  fit <- Krig(pc.mat[my.center.id>0,1:nc], alpha.med.odds[my.center.id>0], m=2, theta=my.theta,
          Distance="rdist", na.rm=TRUE)
  #writePlot("plot_alpha")
  surface(fit, type ="I", col=hc, ylab="PC2", main="alpha",
          las=1, xlab="PC1", extrap=F, axes=T)
  #dev.off()

 
  ## the smooth plot for beta
  fit <- Krig(pc.mat[my.center.id>0,1:nc], beta.med.odds[my.center.id>0], m=2, theta=my.theta,
          Distance="rdist", na.rm=TRUE)
  #writePlot("plot_beta")
  surface(fit, type ="I", col=hc, ylab="PC2", main="beta",
           las=1, xlab="PC1", extrap=F, axes=T)
  dev.off()
  
  list(dic=dic.value, subj.assign=subj.assign, alpha.med.odds=alpha.med.odds, beta.med.odds=beta.med.odds,
       pc.mat=pc.mat[, 1:nc])
}
